/*
GdiPlusRadar.cpp 
(c) 2006 Arne Bartels arne.bartels@t-online.de
Drawing of AI-data with GDI+
*/
#include	"GdiPlusRadar.h"
#include	<wchar.h>
#include	<stdio.h>
#include "math.h"

void	GdiPlusRadar::CheckTooltip	(tAircraftData*	pai_data,double x,double y)
{
	double	dx	=	x-(m_crs_x*m_size_x-m_centre_x),
			dy	=	y-(m_crs_y*m_size_y-m_centre_y),
			dist=sqrt(dx*dx+dy*dy);

	if(pai_data!=NULL)
	{
		//cursor within tags a factor to extend the sensitive area 
		if(dist<=(TAG_SIZE)*2.0*m_symbol_factor)
		{
			//no previous AI near to cursor
			if(m_crs_dist<0
				//this AI is nearer then another
				|| dist < m_crs_dist)
			{
				//remember the distance between cursor and AI picture,
				//there might be one that is nearer
				//then the string is simply overwritten
				m_crs_dist	=	dist;
				memcpy(&m_tooltip_data,pai_data,sizeof(m_tooltip_data));
			}
		}
	}	
}
void	GdiPlusRadar::PadOneAiSymbol	(tAircraftData*	pai_data,FLOAT64	true_hdg,int act_index)
{

	Font		font		(m_symbolfont, m_symbol_factor*FONT_SIZE);

	WCHAR	altstring[100+1]=	L"",
			idstring[100+1]	=	L"";
	float	x				=	0,
			y				=	0;

	int		detail			=	3;

	if(pai_data!=NULL&&m_graphics!=NULL)
	{
	
			
		//select colour to use
		switch(pai_data->tcas_state)
		{
			case	TCAS_OTHER:
				detail	=	m_other_detail;
				break;
			case	TCAS_TA_TRAFFIC:
				detail	=	m_ta_detail;
				break;
			case	TCAS_RA_TRAFFIC:
				detail	=	m_ra_detail;
				break;
			case	TCAS_PROXY_TRAFFIC:
				detail	=	m_proxy_detail;
				break;
			default:
				return;

		}
		

		//calculate x,y pos from bearing and distance
		x	=	(float)(DEG_SIN(pai_data->brg-true_hdg)*pai_data->dist/m_resolution);
		y	=	(float)(-DEG_COS(pai_data->brg-true_hdg)*pai_data->dist/m_resolution);
		

		swprintf_s(altstring,L"%+03d",ROUND(pai_data->dlt_alt/100));
		swprintf_s(idstring,L"%S",pai_data->atc_id);
		
		if(detail && m_dsp_act_cnt<m_max_dsp_act)//display AI type at all
			//and number of displayed aircrafts(m_dsp_act_cnt) is smaller then the maximum (m_max_dsp_act)
		{

			//count displayed aircraft
			m_dsp_act_cnt++;
			//remember highest index so far
			m_max_ind_dsp	=	max(m_max_ind_dsp,act_index);
			if(detail &&	m_pad_symbol)
				PadSymbol(x, y,detail,pai_data,altstring,idstring);
		}

	}
}


void	GdiPlusRadar::DrawOneAiSymbol	(tAircraftData*	pai_data,FLOAT64	true_hdg)
{
	Color		color(0xff,0,0xff,0);


	WCHAR	altstring[100+1]=	L"",
			idstring[100+1]	=	L"";
	float	x				=	0,
			y				=	0;

	int		detail			=	3;

	if(pai_data!=NULL&&m_graphics!=NULL)
	{
	
			
		//select colour to use
		switch(pai_data->tcas_state)
		{
			case	TCAS_OTHER:
   				color.SetValue(m_other_argb);
				detail	=	m_other_detail;
				break;
			case	TCAS_TA_TRAFFIC:
				color.SetValue(m_ta_argb);
				detail	=	m_ta_detail;
				break;
			case	TCAS_RA_TRAFFIC:
				color.SetValue(m_ra_argb);
				detail	=	m_ra_detail;
				break;
			case	TCAS_PROXY_TRAFFIC:
				color.SetValue(m_proxy_argb);
				detail	=	m_proxy_detail;
				break;
			default:
				return;
		}
		



		//calculate x,y pos from bearing and distance
		x	=	(float)(DEG_SIN(pai_data->brg-true_hdg)*pai_data->dist/m_resolution);
		y	=	(float)(-DEG_COS(pai_data->brg-true_hdg)*pai_data->dist/m_resolution);
		

		swprintf_s(altstring,L"%+03d",ROUND(pai_data->dlt_alt/100));
		swprintf_s(idstring,L"%S",pai_data->atc_id);
		
		if(detail)//display AI type at all
		{
			DrawSymbol(x, y,detail,pai_data,true_hdg,color,altstring,idstring);
			//add a tooltip if near to the cursor
			CheckTooltip(pai_data,x,y);
		}

	}
}

void	GdiPlusRadar::DrawAiData(double	true_hdg)
{
	SINT32			ai_num		=	0,
					i			=	0;
	tAircraftData*	aai_data	=	NULL;

	
	if(m_traffic_data==NULL)
		return;
	//get number of available ais
	ai_num		=	m_traffic_data->GetNumAis();
	
	//get ai data
	aai_data	=	m_traffic_data->GetAiData();

	if(aai_data!=NULL)
	{
		
		m_dsp_act_cnt	=	0;
		m_max_ind_dsp	=	0;
		//if background padding is used, pad all necessary pads before drawing!
		//but use this to check how much acft are drawn: check on m_pad_symbol in function
		for(i=1;i<ai_num;i++)
			PadOneAiSymbol(&(aai_data[i]),true_hdg,i);
		//draw each ai, bearing data rotated by own aircraft heading
		//backwards through the list, so nearest aircrafts are drawn last
		for(i=min(m_max_ind_dsp,ai_num-1);i>0;i--)
			DrawOneAiSymbol(&(aai_data[i]),true_hdg);
	}
}
ARGB	GdiPlusRadar::DimColour		(ARGB colour,int brightness)
{
	ARGB ret_val	=	0,
		 red		=	(colour>>RED_SHIFT)&0x0000ff,
		 green		=	(colour>>GREEN_SHIFT)&0x0000ff,
		 blue		=	(colour>>BLUE_SHIFT)&0x0000ff;

	brightness	=	min(0xff,brightness);
	brightness	=	max(0,brightness);

	red			= (ARGB)(red*brightness/255.0);
	green		= (ARGB)(green*brightness/255.0);
	blue		= (ARGB)(blue*brightness/255.0);

	ret_val		= red<<RED_SHIFT | green<<GREEN_SHIFT | blue<<BLUE_SHIFT;


	return	ret_val|ALPHA_MASK;

}

//--------------------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------------------
//public methods
//--------------------------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------------------------
void	GdiPlusRadar::SetMouseCoordinates(double	crs_x,double	crs_y)
{
	m_crs_x	=	crs_x;
	m_crs_y	=	crs_y;
}
void	GdiPlusRadar::SetColours		(int brightness,int inact_colour,int strd_colour,int ta_colour,int ra_colour)
{
	m_other_argb	=	DimColour(inact_colour,brightness);
	m_proxy_argb	=	DimColour(strd_colour,brightness);
	m_ta_argb		=	DimColour(ta_colour,brightness);
	m_ra_argb		=	DimColour(ra_colour,brightness);
}
void	GdiPlusRadar::SetDetails		(int inact_detail,int strd_detail,int ta_detail,int ra_detail,int padsymbol,char * fontname,int max_disp_act)
{
	m_other_detail	=	inact_detail;
	m_proxy_detail	=	strd_detail;
	m_ta_detail		=	ta_detail;
	m_ra_detail		=	ra_detail;
	m_pad_symbol	=	padsymbol;
	if(fontname)
		swprintf_s(m_symbolfont,L"%S",fontname);
	m_max_dsp_act	=	max_disp_act;
}
void	GdiPlusRadar::ScaleScreen(double size_x,double size_y,double scale_x,double scale_y,double centre_x,double centre_y,double range,BOOL fixed_resolution,double	symbol_factor)
{

	if(fixed_resolution)
	{
		m_centre_x		=	(float)(centre_x*scale_x);
		m_centre_y		=	(float)(centre_y*scale_y);
		m_scale_x		=	1;
		m_scale_y		=	1;
		m_size_x		=	(float)(size_x);
		m_size_y		=	(float)(size_y);
	}
	else
	{
		m_scale_x		=	(float)scale_x;
		m_scale_y		=	(float)scale_y;
		m_size_x		=	(float)(size_x/scale_x);
		m_size_y		=	(float)(size_y/scale_y);
		m_centre_x		=	(float)centre_x;
		m_centre_y		=	(float)centre_y;
	}


	m_resolution	=	(float)(range*2.0/min(m_size_x,m_size_y));

	
	m_symbol_factor	=	(float)symbol_factor;
}

//--------------------------------------------------------------------------------------------------------
//image method
//--------------------------------------------------------------------------------------------------------
bool	GdiPlusRadar::Draw( HDC hdc,double	true_hdg,BOOL antialias)
{	
	GraphicsPath	circlemask;
	SolidBrush		blackbrush		(Color(0xff,0,0,0));

	if(hdc==NULL)			
		return	false;
	
	SAFE_DELETE(m_graphics);
	m_graphics	=	new Graphics(hdc);

	if(m_graphics==NULL)	
		return	false;

	
	//scale the drawing to fit the bitmap resolution
	//from now on all drawing coordinates are relative to the bitmap resolution
	//you could omit it here and take the actual size of the gauge later in account
	//when each element is drawn, but this is simpler, it might cost a bit of processor time though

	m_graphics->ScaleTransform(m_scale_x,m_scale_y);

	if(antialias)
	{
		//make all drawings antialiased
		m_graphics->SetSmoothingMode(SmoothingModeAntiAlias);

		//make all texts antialiased
		m_graphics->SetTextRenderingHint(TextRenderingHintAntiAliasGridFit);
	}
					
	//whipe the blackboard
	m_graphics->FillRectangle(&blackbrush,RectF(0,0,m_size_x,m_size_y));
/*
//A bounding rect and crosshair for proper alignement
Pen			pen(Color(255,255,255),1);
m_graphics->DrawRectangle(&pen,RectF(0,0,m_size_x,m_size_y));
m_graphics->DrawLine(&pen, PointF(0,0),PointF(m_size_x,m_size_y));
m_graphics->DrawLine(&pen, PointF(0,m_size_y/2),PointF(m_size_x,m_size_y/2));
m_graphics->DrawLine(&pen, PointF(m_size_x/2,0),PointF(m_size_x/2,m_size_y));
m_graphics->DrawLine(&pen, PointF(m_size_x,0),PointF(0,m_size_y));
*/	
	//shift the coordinate system to the centre
	m_graphics->TranslateTransform(m_centre_x,m_centre_y);


	//preset shortest distance between cursor and AI with an invalid value, to indicate that none is found yet
	m_crs_dist	=	-1.0f;



	memset(&m_tooltip_data,0,sizeof(m_tooltip_data));
	DrawAiData(true_hdg);
		
	return	true;
}

void	GdiPlusRadar::SetCatRules		(double other_min_ralt,double other_above_alt,double other_below_alt,double proxy_range,double proxy_above_alt,double proxy_below_alt)
{
	if(m_traffic_data) 
		m_traffic_data->SetCatRules(other_min_ralt,other_above_alt,other_below_alt,proxy_range,proxy_above_alt,proxy_below_alt);
}

void	GdiPlusRadar::QueryAiData(	UINT32 maxnum_act,FLOAT64 max_range_nm,
									FLOAT64 time_sep_ta,FLOAT64 time_sep_ra,
									FLOAT64 dist_sep_ta,FLOAT64 dist_sep_ra,
									FLOAT64 alt_sep_ta,	FLOAT64 alt_sep_ra)
{
	if(m_traffic_data!=NULL)
		m_traffic_data->QueryData(	maxnum_act,		max_range_nm,
									time_sep_ta,	time_sep_ra,
									dist_sep_ta,	dist_sep_ra,
									alt_sep_ta,		alt_sep_ra);

};



tAircraftData*	GdiPlusRadar::GetTooltipData	(void)	
{
	return &m_tooltip_data;
};
tAircraftData*	GdiPlusRadar::GetAdvisoryData	(void)	
{
	if(m_traffic_data) 
		return m_traffic_data->GetAdvisoryData();
	return 0;
};





//--------------------------------------------------------------------------------------------------------
//constructor/destructor
//--------------------------------------------------------------------------------------------------------
GdiPlusRadar::GdiPlusRadar()
{
	GdiplusStartupInput startupinput(NULL,FALSE,FALSE); 

	
	m_graphics		=	NULL;
	m_gdiplustoken	=	0;
	wcscpy_s(m_symbolfont,FONT_TYPE);


	SetMouseCoordinates(0,0);
	m_crs_dist		=	-1.0;

	m_traffic_data	=	new	TrafficData();
	memset(&m_tooltip_data,0,sizeof(m_tooltip_data));
	
	m_other_argb	=	0xFFF0F0F0;
	m_proxy_argb	=	0xFF00FF00;
	m_ta_argb		=	0xFFFFFF00;
	m_ra_argb		=	0xFFFF0000;
	
	m_other_detail	=	3;
	m_proxy_detail	=	3;
	m_ta_detail		=	3;
	m_ra_detail		=	3;
	m_pad_symbol	=	FALSE;

	m_size_x		=	HALF_WIDTH*2;
	m_size_y		=	HALF_HEIGHT*2;
	m_scale_x		=	1;
	m_scale_y		=	1;

	m_resolution	=	INITIAL_RANGE/HALF_WIDTH;
	m_max_dsp_act	=	MAX_NUM_AIRCRAFT;
	m_dsp_act_cnt	=	0;
	m_max_ind_dsp	=	0;

	GdiplusStartup( &m_gdiplustoken, &startupinput, NULL ); 

}

GdiPlusRadar::~GdiPlusRadar	(void)
{
	SAFE_DELETE(m_traffic_data);
	SAFE_DELETE(m_graphics);
	if(m_gdiplustoken)
		GdiplusShutdown( m_gdiplustoken ); 
}
//--------------------------------------------------------------------------------------------------------

void	RadarScreen::PadSymbol(float x, float y,int detail,tAircraftData*	pai_data,WCHAR	*altstring,WCHAR	*idstring)
{
	SolidBrush		padbrush(Color(0xff,0x10,0x10,0x10));
	RectF			boundRect;
	StringFormat	stringFormat;
	stringFormat.SetAlignment(StringAlignmentCenter);
	Font			font	(m_symbolfont, m_symbol_factor*FONT_SIZE);

	if(m_graphics	==	NULL)	return;
	if(pai_data		==	NULL)	return;
	if(altstring	==	NULL)	return;
	if(idstring		==	NULL)	return;

	if((detail&DETAIL_SYMBOL_OUTLINE) || (detail&DETAIL_SYMBOL_FILLED))
		m_graphics->FillRectangle(&padbrush, x-m_symbol_factor*PAD_SIZE,y-m_symbol_factor*PAD_SIZE,2.0f*m_symbol_factor*PAD_SIZE,2.0f*m_symbol_factor*PAD_SIZE);

	m_graphics->MeasureString(altstring,(int)wcslen(altstring),&font,PointF(x+m_symbol_factor*ABVSTR_OFFSET_X,y+m_symbol_factor*ABVSTR_OFFSET_Y),&stringFormat, &boundRect);		
	if(detail&DETAIL_ALTITUDE)// => draw relative altitude above the symbol
		m_graphics->FillRectangle(&padbrush, boundRect);
	m_graphics->MeasureString(idstring,(int)wcslen(idstring),&font,PointF(x+m_symbol_factor*BLWSTR_OFFSET_X,y+m_symbol_factor*BLWSTR_OFFSET_Y),&stringFormat, &boundRect);
	if(detail&DETAIL_ID)// => draw id below the symbol
		m_graphics->FillRectangle(&padbrush, boundRect);
}


void	RadarScreen::DrawSymbol(float x, float y,int detail,tAircraftData*	pai_data,FLOAT64	true_hdg,Color& color,WCHAR	*altstring,WCHAR	*idstring)
{
		SolidBrush		brush	(color);
		Pen				pen		(color,m_symbol_factor*PEN_WIDTH);
		Font			font	(m_symbolfont, m_symbol_factor*FONT_SIZE);

		StringFormat	stringFormat;
		stringFormat.SetAlignment(StringAlignmentCenter);

		if(m_graphics	==	NULL)	return;
		if(pai_data		==	NULL)	return;
		if(altstring	==	NULL)	return;
		if(idstring		==	NULL)	return;

		if(detail&DETAIL_DOT)//draw dot at aircraft position
			m_graphics->FillEllipse(&brush, x-m_symbol_factor*DOT_SIZE,y-m_symbol_factor*DOT_SIZE,2.0f*m_symbol_factor*DOT_SIZE,2.0f*m_symbol_factor*DOT_SIZE);
		if(detail&DETAIL_SYMBOL_OUTLINE)//draw circle at aircraft position
		{
			m_graphics->DrawEllipse(&pen, x-m_symbol_factor*TAG_SIZE,y-m_symbol_factor*TAG_SIZE,2.0f*m_symbol_factor*TAG_SIZE,2.0f*m_symbol_factor*TAG_SIZE);
			//draw track angle if gs>0kt
			if(pai_data->grnd_spd>0)
				m_graphics->DrawLine(&pen, PointF(x,y),PointF((float)(x+m_symbol_factor*TRACK_ANGLE_LEN*DEG_SIN(pai_data->trck_angl-true_hdg)),(float)(y-m_symbol_factor*TRACK_ANGLE_LEN*DEG_COS(pai_data->trck_angl-true_hdg))));
		}
		if(detail&DETAIL_SYMBOL_FILLED)
			m_graphics->FillEllipse(&brush, x-m_symbol_factor*TAG_SIZE,y-m_symbol_factor*TAG_SIZE,2.0f*m_symbol_factor*TAG_SIZE,2.0f*m_symbol_factor*TAG_SIZE);
					
		if(detail&DETAIL_ALTITUDE)// => draw relative altitude above the symbol
			m_graphics->DrawString(altstring,(int)wcslen(altstring),&font,PointF(x+m_symbol_factor*ABVSTR_OFFSET_X,y+m_symbol_factor*ABVSTR_OFFSET_Y),&stringFormat,&brush);
			
		if(detail&DETAIL_ID)// => draw id below the symbol
			m_graphics->DrawString(idstring,(int)wcslen(idstring),&font,PointF(x+m_symbol_factor*BLWSTR_OFFSET_X,y+m_symbol_factor*BLWSTR_OFFSET_Y),&stringFormat,&brush);

}

void	TCASScreen::PadSymbol(float x, float y,int detail,tAircraftData*	pai_data,WCHAR	*altstring,WCHAR	*idstring)
{
	int				diff_alt	=	0;
	SolidBrush		padbrush(Color(0xff,0x10,0x10,0x10));
	RectF			boundRect;
	Font			font(m_symbolfont, m_symbol_factor*FONT_SIZE);

	StringFormat	stringFormat;
	stringFormat.SetAlignment(StringAlignmentCenter);

	if(m_graphics	==	NULL)	return;
	if(pai_data		==	NULL)	return;
	if(altstring	==	NULL)	return;
	if(idstring		==	NULL)	return;
	diff_alt	=	ROUND(pai_data->dlt_alt/100);

	if((detail&DETAIL_SYMBOL_OUTLINE) || (detail&DETAIL_SYMBOL_FILLED))
		m_graphics->FillRectangle(&padbrush, x-m_symbol_factor*PAD_SIZE,y-m_symbol_factor*PAD_SIZE,2.0f*m_symbol_factor*PAD_SIZE,2.0f*m_symbol_factor*PAD_SIZE);

			
	if(detail&DETAIL_ALTITUDE)// => pad relative altitude above/below symbol
	{
		if(diff_alt<0)
			m_graphics->MeasureString(altstring,(int)wcslen(altstring),&font,PointF(x+m_symbol_factor*BLWSTR_OFFSET_X,y+m_symbol_factor*BLWSTR_OFFSET_Y),&stringFormat, &boundRect);		
		else
			m_graphics->MeasureString(altstring,(int)wcslen(altstring),&font,PointF(x+m_symbol_factor*ABVSTR_OFFSET_X,y+m_symbol_factor*ABVSTR_OFFSET_Y),&stringFormat, &boundRect);		
		m_graphics->FillRectangle(&padbrush, boundRect);
		
		if(fabs(pai_data->vs)>500)//pad up/down arrow
			m_graphics->FillRectangle(&padbrush, x+m_symbol_factor*(POINTER_OFFSET-PAD_SIZE/2.0f),y-m_symbol_factor*PAD_SIZE,m_symbol_factor*PAD_SIZE,2.0f*m_symbol_factor*PAD_SIZE);
	}
	if(detail&DETAIL_ID)// => pad id below/above symbol
	{
		if(diff_alt<0)
			m_graphics->MeasureString(idstring,(int)wcslen(idstring),&font,PointF(x+m_symbol_factor*ABVSTR_OFFSET_X,y+m_symbol_factor*ABVSTR_OFFSET_Y),&stringFormat,&boundRect);		
		else
			m_graphics->MeasureString(idstring,(int)wcslen(idstring),&font,PointF(x+m_symbol_factor*BLWSTR_OFFSET_X,y+m_symbol_factor*BLWSTR_OFFSET_Y),&stringFormat,&boundRect);		
		m_graphics->FillRectangle(&padbrush, boundRect);
	}
}
void	TCASScreen::DrawSymbol(float x, float y,int detail,tAircraftData*	pai_data,FLOAT64	true_hdg,Color& color,WCHAR	*altstring,WCHAR	*idstring)
{
		int				diff_alt	=	0;
		SolidBrush		brush	(color);
		Pen				pen		(color,m_symbol_factor*PEN_WIDTH);
		Font			font	(m_symbolfont, m_symbol_factor*FONT_SIZE);

		StringFormat	stringFormat;
		stringFormat.SetAlignment(StringAlignmentCenter);

		RectF			rect	(	x-m_symbol_factor*TAG_SIZE, 
									y-m_symbol_factor*TAG_SIZE, 
									2.0f*m_symbol_factor*TAG_SIZE, 
									2.0f*m_symbol_factor*TAG_SIZE);
		PointF			points[4] = {
										PointF(x,y-m_symbol_factor*TAG_SIZE), 
										PointF(x+m_symbol_factor*TAG_SIZE,y),
										PointF(x,y+m_symbol_factor*TAG_SIZE),
										PointF(x-m_symbol_factor*TAG_SIZE,y)
									};
		
		
		if(m_graphics	==	NULL)	return;
		if(pai_data		==	NULL)	return;
		if(altstring	==	NULL)	return;
		if(idstring		==	NULL)	return;
		diff_alt	=	ROUND(pai_data->dlt_alt/100);


		if(detail&DETAIL_DOT)//draw dot at aircraft position
			m_graphics->FillEllipse(&brush, x-m_symbol_factor*DOT_SIZE,y-m_symbol_factor*DOT_SIZE,2.0f*m_symbol_factor*DOT_SIZE,2.0f*m_symbol_factor*DOT_SIZE);
			

		if(detail&DETAIL_SYMBOL_OUTLINE)//outline symbol at aircraft position
		{
			switch(pai_data->tcas_state)
			{
				case	TCAS_OTHER:
				case	TCAS_PROXY_TRAFFIC:
					m_graphics->DrawPolygon(&pen,points,4);//diamond
					break;
				case	TCAS_TA_TRAFFIC:
					m_graphics->DrawEllipse(&pen,rect);//circle
					break;
				case	TCAS_RA_TRAFFIC:
					m_graphics->DrawRectangle(&pen,rect);//square
					break;
			}
		}

		if(detail&DETAIL_SYMBOL_FILLED)//fill symbol at aircraft position
		{
			switch(pai_data->tcas_state)
			{
				case	TCAS_OTHER:
				case	TCAS_PROXY_TRAFFIC:
					m_graphics->FillPolygon(&brush,points,4);
					break;
				case	TCAS_TA_TRAFFIC:
					m_graphics->FillEllipse(&brush,rect);
					break;
				case	TCAS_RA_TRAFFIC:
					m_graphics->FillRectangle(&brush,rect);
					break;
			}
		}

		if(detail&DETAIL_ALTITUDE)// => draw relative altitude above/below symbol
		{
			if(diff_alt<0)
				m_graphics->DrawString(altstring,(int)wcslen(altstring),&font,PointF(x+m_symbol_factor*BLWSTR_OFFSET_X,y+m_symbol_factor*BLWSTR_OFFSET_Y),&stringFormat,&brush);
			else
				m_graphics->DrawString(altstring,(int)wcslen(altstring),&font,PointF(x+m_symbol_factor*ABVSTR_OFFSET_X,y+m_symbol_factor*ABVSTR_OFFSET_Y),&stringFormat,&brush);

			if(fabs(pai_data->vs)>500)//draw up/down arrow
			{
				m_graphics->DrawLine(
					&pen, 
					PointF(x+m_symbol_factor*POINTER_OFFSET,y-m_symbol_factor*TAG_SIZE),
					PointF(x+m_symbol_factor*POINTER_OFFSET,y+m_symbol_factor*TAG_SIZE)
					);
				if(pai_data->vs>0)
				{
					m_graphics->DrawLine(
											&pen, 
											PointF(x+m_symbol_factor*(POINTER_OFFSET-TAG_SIZE*0.5f),y-m_symbol_factor*TAG_SIZE*0.5f),
											PointF(x+m_symbol_factor*POINTER_OFFSET,				y-m_symbol_factor*TAG_SIZE)
											);
					m_graphics->DrawLine(
											&pen, 
											PointF(x+m_symbol_factor*(POINTER_OFFSET+TAG_SIZE*0.5f),y-m_symbol_factor*TAG_SIZE*0.5f),
											PointF(x+m_symbol_factor*POINTER_OFFSET,				y-m_symbol_factor*TAG_SIZE)
											);
				}
				if(pai_data->vs<0)
				{
					m_graphics->DrawLine(
											&pen, 
											PointF(x+m_symbol_factor*(POINTER_OFFSET-TAG_SIZE*0.5f),y+m_symbol_factor*TAG_SIZE*0.5f),
											PointF(x+m_symbol_factor*POINTER_OFFSET,				y+m_symbol_factor*TAG_SIZE)
											);
					m_graphics->DrawLine(
											&pen, 
											PointF(x+m_symbol_factor*(POINTER_OFFSET+TAG_SIZE*0.5f),y+m_symbol_factor*TAG_SIZE*0.5f),
											PointF(x+m_symbol_factor*POINTER_OFFSET,				y+m_symbol_factor*TAG_SIZE)
											);
				}
			}

		}
	
		if(detail&DETAIL_ID)// => draw id below/above symbol
		{
			if(diff_alt<0)
				m_graphics->DrawString(idstring,(int)wcslen(idstring),&font,PointF(x+m_symbol_factor*ABVSTR_OFFSET_X,y+m_symbol_factor*ABVSTR_OFFSET_Y),&stringFormat,&brush);
			else
				m_graphics->DrawString(idstring,(int)wcslen(idstring),&font,PointF(x+m_symbol_factor*BLWSTR_OFFSET_X,y+m_symbol_factor*BLWSTR_OFFSET_Y),&stringFormat,&brush);
		}
}
