/*
TrafficData.h 
(c) 2004 Arne Bartels arne.bartels@nwn.de
*/
#ifndef	TRAFFICDATA_H
#define	TRAFFICDATA_H

//#include	"fs9gauges.h"

#define		MAX_CHARS_ATC_ID	10


#if defined(_MSC_VER) 
	#if (_MSC_VER<1400) 
		#ifndef	swprintf_s
			#define	swprintf_s swprintf
		#endif//	swprintf_s

		#ifndef	wcscpy_s
			#define	wcscpy_s wcscpy
		#endif//	wcscpy_s

		#ifndef	strncpy_s
			#define	strncpy_s strncpy
		#endif//	strncpy_s

		#ifndef	sprintf_s
			#define	sprintf_s sprintf
		#endif//	sprintf_s

		#ifndef	strcpy_s
			#define	strcpy_s(dest,destsize,src) strcpy(dest,src)
		#endif//	strcpy_s
	#endif// (_MSC_VER<1400) 
#endif// defined(_MSC_VER) 


#define		TCAS_OFF					(-2)
#define		TCAS_OTHER					(-1)
#define		TCAS_PROXY_TRAFFIC			0
#define		TCAS_TA_TRAFFIC				1
#define		TCAS_RA_TRAFFIC				2

#define		CLEAR_OF_GROUND	50
#define		OTHER_DIFFALT	2700
#define		PROXY_RANGE		6
#define		PROXY_DIFFALT	1200


typedef	struct	sAircraftData
{
//original data
	UINT32		id;
	FLOAT64		lat;
	FLOAT64		lon;
	FLOAT64		alt;
	FLOAT64		vs;
	FLOAT64		ns;
	FLOAT64		es;
	char		atc_id[MAX_CHARS_ATC_ID+1];
	FLOAT64		comb;
	FLOAT64		is_user;
	FLOAT64		alt_agl;

//derived data
	FLOAT64		brg;
	FLOAT64		dist;
	FLOAT64		slnt_rng;
	FLOAT64		rng_rate;
	FLOAT64		dlt_alt;
//	FLOAT64		dlt_vs;
	FLOAT64		trck_angl;
	FLOAT64		grnd_spd;

	int		tcas_state;

}
tAircraftData;

EXTERN_C int compare_ids( const void *arg1, const void *arg2 );


class TrafficData  
{

private:

	IPanelCCallback*	m_ppanel_cb;
	ITrafficInfo*		m_ptraffic_info;

	ENUM				m_var_lat;
	ENUM				m_var_lon;
	ENUM				m_var_alt;
	ENUM				m_var_ind_alt;
	ENUM				m_var_vs;
	ENUM				m_var_ns;
	ENUM				m_var_es;
	ENUM				m_var_hdg_true;
	ENUM				m_var_tail_nr;
	ENUM				m_var_comb;
	ENUM				m_var_is_user;
	ENUM				m_var_alt_agl;

	ENUM				m_unit_feet;
	ENUM				m_unit_bool;
	ENUM				m_unit_deg;
	ENUM				m_unit_kts;
	ENUM				m_unit_fpm;

	tAircraftData*		m_ai_data;
	
	tAircraftData		m_closest_threat;
	
	UINT32				m_num_ai_ids;

	MODULE_VAR			m_elapsed_secs;

	double				m_other_min_ralt;
	double				m_other_above_alt;
	double				m_other_below_alt;
	double				m_proxy_range;
	double				m_proxy_above_alt;
	double				m_proxy_below_alt;

	double				m_time_sep_ta;
	double				m_time_sep_ra;
	double				m_distance_sep_ta;
	double				m_distance_sep_ra;
	double				m_altitude_sep_ta;
	double				m_altitude_sep_ra;
	

private:

	void			GetAircraftData		(UINT32 ai_id,tAircraftData*	pai_data);
	void			DeriveData			(UINT32 ai_num,tAircraftData*	pai_data,UINT32 old_ai_num,tAircraftData*	old_pai_data);
	void			DifferentiateData	(tAircraftData*	pai_data,UINT32 old_ai_num,tAircraftData*	old_pai_data,FLOAT64	time_diff,FLOAT64 plane_vs);
	
public:

	UINT32			QueryData			(UINT32 maxnum_act,FLOAT64 max_range,
										FLOAT64 time_sep_ta,FLOAT64 time_sep_ra,
										FLOAT64 dist_sep_ta,FLOAT64 dist_sep_ra,
										FLOAT64 alt_sep_ta,FLOAT64 alt_sep_ra);

	void			SetCatRules			(double other_min_ralt,double other_above_alt,double other_below_alt,double proxy_range,double proxy_above_alt,double proxy_below_alt);

	UINT32			GetNumAis			(void){return	m_num_ai_ids;};
	tAircraftData*	GetAiData			(void){return	m_ai_data;};
	//int				GetAdvisory			(void){return	m_adv_state;};
	tAircraftData*	GetAdvisoryData		(void){return	&m_closest_threat;};


	
	TrafficData			(void);
	virtual			~TrafficData		(void);

};

//insert in your gauges.h
typedef enum XML_TOKEN
{
		XML_NONE,
		XML_ANGLE_OF_ATTACK_INDICATOR,
		XML_GUN_AMMO,
		XML_CANNON_AMMO,
		XML_ROCKET_AMMO,
		XML_BOMB_AMMO,
		XML_LIGHT_STATES,
		XML_LIGHT_PANEL,
		XML_LIGHT_STROBE,
		XML_LIGHT_LANDING,
		XML_STROBE_FLASH,
		XML_LIGHT_TAXI,
		XML_LIGHT_BEACON,
		XML_LIGHT_NAV,
		XML_LIGHT_LOGO,
		XML_LIGHT_WING,
		XML_LIGHT_RECOGNITION,
		XML_LIGHT_CABIN,
		XML_LANDING_LIGHT_PBH,
		XML_CENTER_WHEEL_RPM,
		XML_LEFT_WHEEL_RPM,
		XML_RIGHT_WHEEL_RPM,
		XML_TOTAL_WORLD_VELOCITY,
		XML_GROUND_VELOCITY,
		XML_AIRSPEED_TRUE,
		XML_AIRSPEED_INDICATED,
		XML_AIRSPEED_SELECT_INDICATED_OR_TRUE,
		XML_AIRSPEED_TRUE_CALIBRATE,
		XML_AIRSPEED_BARBER_POLE,
		XML_AIRSPEED_MACH,
		XML_VERTICAL_SPEED,
		XML_MACH_MAX_OPERATE,
		XML_STALL_WARNING,
		XML_OVERSPEED_WARNING,
		XML_BARBER_POLE_MACH,
		XML_VELOCITY_BODY_X,
		XML_VELOCITY_BODY_Y,
		XML_VELOCITY_BODY_Z,
		XML_VELOCITY_WORLD_X,
		XML_VELOCITY_WORLD_Y,
		XML_VELOCITY_WORLD_Z,
		XML_RELATIVE_WIND_VELOCITY_BODY_X,
		XML_RELATIVE_WIND_VELOCITY_BODY_Y,
		XML_RELATIVE_WIND_VELOCITY_BODY_Z,
		XML_DESIGN_SPEED_VS0,
		XML_DESIGN_SPEED_VS1,
		XML_DESIGN_SPEED_VC,
		XML_MIN_DRAG_VELOCITY,
		XML_PLANE_LATITUDE,
		XML_PLANE_LONGITUDE,
		XML_PLANE_ALTITUDE,
		XML_PLANE_ALT_ABOVE_GROUND,
		XML_PLANE_PITCH_DEGREES,
		XML_PLANE_BANK_DEGREES,
		XML_PLANE_HEADING_DEGREES_MAGNETIC,
		XML_PLANE_HEADING_DEGREES_TRUE,
		XML_INDICATED_ALTITUDE,
		XML_PRESSURE_ALTITUDE,
		XML_KOHLSMAN_SETTING_MB,
		XML_KOHLSMAN_SETTING_HG,
		XML_ATTITUDE_INDICATOR_PITCH_DEGREES,
		XML_ATTITUDE_INDICATOR_BANK_DEGREES,
		XML_ATTITUDE_BARS_POSITION,
		XML_ATTITUDE_CAGE,
		XML_MAGVAR,
		XML_WISKEY_COMPASS_INDICATION_DEGREES,
		XML_PLANE_HEADING_DEGREES_GYRO,
		XML_GYRO_DRIFT_ERROR,
		XML_DELTA_HEADING_RATE,
		XML_GROUND_ALTITUDE,
		XML_SIM_ON_GROUND,
		XML_TURN_COORDINATOR_BALL,
		XML_YOKE_Y_POSITION,
		XML_YOKE_Y_INDICATOR,
		XML_YOKE_X_POSITION,
		XML_YOKE_X_INIDICATOR,
		XML_AILERON_POSITION,
		XML_RUDDER_PEDAL_POSITION,
		XML_RUDDER_PEDAL_INDICATOR,
		XML_RUDDER_POSITION,
		XML_ELEVATOR_POSITION,
		XML_ELEVATOR_TRIM_POSITION,
		XML_ELEVATOR_TRIM_INDICATOR,
		XML_ELEVATOR_TRIM_PCT,
		XML_BRAKE_LEFT_POSITION,
		XML_BRAKE_RIGHT_POSITION,
		XML_BRAKE_INDICATOR,
		XML_BRAKE_PARKING_POSITION,
		XML_BRAKE_PARKING_INDICATOR,
		XML_SPOILERS_ARMED,
		XML_SPOILERS_HANDLE_POSITION,
		XML_SPOILERS_LEFT_POSITION,
		XML_SPOILERS_RIGHT_POSITION,
		XML_FLAPS_NUM_HANDLE_POSITIONS,
		XML_FLAPS_HANDLE_PERCENT,
		XML_FLAPS_HANDLE_INDEX,
		XML_TRAILING_EDGE_FLAPS_LEFT_PERCENT,
		XML_TRAILING_EDGE_FLAPS_RIGHT_PERCENT,
		XML_LEADING_EDGE_FLAPS_LEFT_PERCENT,
		XML_LEADING_EDGE_FLAPS_RIGHT_PERCENT,
		XML_TRAILING_EDGE_FLAPS_LEFT_ANGLE,
		XML_TRAILING_EDGE_FLAPS_RIGHT_ANGLE,
		XML_LEADING_EDGE_FLAPS_LEFT_ANGLE,
		XML_LEADING_EDGE_FLAPS_RIGHT_ANGLE,
		XML_FLAP_POSITION_SET,
		XML_IS_GEAR_RETRACTABLE,
		XML_IS_GEAR_WHEELS,
		XML_IS_GEAR_SKIS,
		XML_IS_GEAR_FLOATS,
		XML_IS_GEAR_SKIDS,
		XML_GEAR_HANDLE_POSITION,
		XML_GEAR_CENTER_POSITION,
		XML_GEAR_LEFT_POSITION,
		XML_GEAR_RIGHT_POSITION,
		XML_GEAR_TAIL_POSITION,
		XML_GEAR_AUX_POSITION,
		XML_GEAR_POSITION,
		XML_GEAR_ANIMATION_POSITION,
		XML_GEAR_TOTAL_PCT_EXTENDED,
		XML_TAILWHEEL_LOCK_ON,
		XML_COWL_FLAPS,
		XML_AVIONICS_MASTER_SWITCH,
		XML_PANEL_AUTO_FEATHER_SWITCH,
		XML_PANEL_ANTI_ICE_SWITCH,
		XML_AUTO_BRAKE_SWITCH_CB,
		XML_WATER_RUDDER_HANDLE_POSITION,
		XML_WATER_LEFT_RUDDER_EXTENDED,
		XML_WATER_RIGHT_RUDDER_EXTENDED,
		XML_GEAR_CENTER_STEER_ANGLE,
		XML_GEAR_LEFT_STEER_ANGLE,
		XML_GEAR_RIGHT_STEER_ANGLE,
		XML_GEAR_AUX_STEER_ANGLE,
		XML_GEAR_STEER_ANGLE,
		XML_WATER_LEFT_RUDDER_STEER_ANGLE,
		XML_WATER_RIGHT_RUDDER_STEER_ANGLE,
		XML_GEAR_CENTER_STEER_ANGLE_PCT,
		XML_GEAR_LEFT_STEER_ANGLE_PCT,
		XML_GEAR_RIGHT_STEER_ANGLE_PCT,
		XML_GEAR_AUX_STEER_ANGLE_PCT,
		XML_GEAR_STEER_ANGLE_PCT,
		XML_WATER_LEFT_RUDDER_STEER_ANGLE_PCT,
		XML_WATER_RIGHT_RUDDER_STEER_ANGLE_PCT,
		XML_ELEVATOR_DEFLECTION,
		XML_ELEVATOR_DEFLECTION_PCT,
		XML_AILERON_LEFT_DEFLECTION,
		XML_AILERON_LEFT_DEFLECTION_PCT,
		XML_AILERON_RIGHT_DEFLECTION,
		XML_AILERON_RIGHT_DEFLECTION_PCT,
		XML_AILERON_AVERAGE_DEFLECTION,
		XML_AILERON_TRIM,
		XML_AILERON_TRIM_PCT,
		XML_RUDDER_DEFLECTION,
		XML_RUDDER_DEFLECTION_PCT,
		XML_RUDDER_TRIM,
		XML_RUDDER_TRIM_PCT,
		XML_PROP_SYNC_ACTIVE,
		XML_INCIDENCE_ALPHA,
		XML_INCIDENCE_BETA,
		XML_CG_PERCENT,
		XML_CG_PERCENT_LATERAL,
		XML_CG_AFT_LIMIT,
		XML_CG_FWD_LIMIT,
		XML_CG_MAX_MACH,
		XML_CG_MIN_MACH,
		XML_PAYLOAD_STATION_WEIGHT,
		XML_PAYLOAD_STATION_NAME,
		XML_PAYLOAD_STATION_COUNT,
		XML_ELEVON_DEFLECTION,
		XML_FOLDING_WING_LEFT_PERCENT,
		XML_FOLDING_WING_RIGHT_PERCENT,
		XML_CANOPY_OPEN,
		XML_TAILHOOK_POSITION,
		XML_EXIT_OPEN,
		XML_RADIO_HEIGHT,
		XML_DECISION_HEIGHT,
		XML_TOTAL_WEIGHT,
		XML_MAX_GROSS_WEIGHT,
		XML_EMPTY_WEIGHT,
		XML_AUTOPILOT_MASTER,
		XML_AUTOPILOT_WING_LEVELER,
		XML_AUTOPILOT_NAV1_LOCK,
		XML_AUTOPILOT_HEADING_LOCK,
		XML_AUTOPILOT_HEADING_LOCK_DIR,
		XML_AUTOPILOT_ALTITUDE_LOCK,
		XML_AUTOPILOT_ALTITUDE_LOCK_VAR,
		XML_AUTOPILOT_ATTITUDE_HOLD,
		XML_AUTOPILOT_GLIDESLOPE_HOLD,
		XML_AUTOPILOT_APPROACH_HOLD,
		XML_AUTOPILOT_BACKCOURSE_HOLD,
		XML_AUTOPILOT_YAW_DAMPER,
		XML_AUTOPILOT_AIRSPEED_HOLD,
		XML_AUTOPILOT_AIRSPEED_HOLD_VAR,
		XML_AUTOPILOT_MACH_HOLD,
		XML_AUTOPILOT_MACH_HOLD_VAR,
		XML_AUTOPILOT_VERTICAL_HOLD,
		XML_AUTOPILOT_VERTICAL_HOLD_VAR,
		XML_AUTOPILOT_ALTITUDE_MANUALLY_TUNABLE,
		XML_AUTOPILOT_HEADING_MANUALLY_TUNABLE,
		XML_AUTOPILOT_THROTTLE_ARM,
		XML_AUTOPILOT_TAKEOFF_POWER_ACTIVE,
		XML_AUTOPILOT_RPM_HOLD_VAR,
		XML_AUTOPILOT_SPEED_SETTING,
		XML_AUTOPILOT_AIRSPEED_ACQUISITION,
		XML_AUTOPILOT_AIRSPEED_HOLD_CURRENT,
		XML_AUTOPILOT_MAX_SPEED_HOLD,
		XML_AUTOPILOT_CRUISE_SPEED_HOLD,
		XML_AUTOPILOT_FLIGHT_DIRECTOR_ACTIVE,
		XML_AUTOPILOT_FLIGHT_DIRECTOR_PITCH,
		XML_AUTOPILOT_FLIGHT_DIRECTOR_BANK,
		XML_AUTOPILOT_PITCH_HOLD_REF,
		XML_AUTOPILOT_NAV_SELECTED,
		XML_GPS_DRIVES_NAV1,
		XML_AUTOTHROTTLE_ACTIVE,
		XML_ENGINE_CONTROL_SELECT,
		XML_NUMBER_OF_ENGINES,
		XML_PROPELLER_ADVANCED_SELECTION,
		XML_THROTTLE_LOWER_LIMIT,
		XML_OIL_AMOUNT,
		XML_ENGINE_PRIMER,
		XML_ENGINE_TYPE,
		XML_ENG_RPM_ANIMATION_PERCENT,
		XML_PROP_RPM,
		XML_PROP_MAX_RPM_PERCENT,
		XML_PROP_THRUST,
		XML_PROP_BETA,
		XML_PROP_FEATHERING_INHIBIT,
		XML_PROP_FEATHERED,
		XML_PROP_SYNC_DELTA_LEVER,
		XML_PROP_AUTO_FEATHER_ARMED,
		XML_PROP_FEATHER_SWITCH,
		XML_PROP_AUTO_CRUISE_ACTIVE,
		XML_MASTER_IGNITION_SWITCH,
		XML_ENG_COMBUSTION,
		XML_OLD_ENG_STARTER,
		XML_ENG_N1_RPM,
		XML_ENG_N2_RPM,
		XML_ENG_FUEL_FLOW_GPH,
		XML_ENG_FUEL_FLOW_PPH,
		XML_ENG_FUEL_FLOW_PPH_SSL,
		XML_ENG_TORQUE,
		XML_ENG_ANTI_ICE,
		XML_ENG_PRESSURE_RATIO,
		XML_ENG_PRESSURE_RATIO_GES,
		XML_ENG_EXHAUST_GAS_TEMPERATURE,
		XML_ENG_EXHAUST_GAS_TEMPERATURE_GES,
		XML_ENG_CYLINDER_HEAD_TEMPERATURE,
		XML_ENG_OIL_TEMPERATURE,
		XML_ENG_OIL_PRESSURE,
		XML_ENG_OIL_QUANTITY,
		XML_ENG_HYDRAULIC_PRESSURE,
		XML_ENG_HYDRAULIC_QUANTITY,
		XML_ENG_MANIFOLD_PRESSURE,
		XML_ENG_VIBRATION,
		XML_ENG_RPM_SCALER,
		XML_ENG_TURBINE_TEMPERATURE,
		XML_ENG_TORQUE_PERCENT,
		XML_ENG_FUEL_PRESSURE,
		XML_ENG_ELECTRICAL_LOAD,
		XML_ENG_TRANSMISSION_PRESSURE,
		XML_ENG_TRANSMISSION_TEMPERATURE,
		XML_ENG_ROTOR_RPM,
		XML_ENG_FUEL_FLOW_BUG_POSITION,
		XML_ENG_MAX_RPM,
		XML_ENG_ON_FIRE,
		XML_GENERAL_ENG_COMBUSTION,
		XML_GENERAL_ENG_MASTER_ALTERNATOR,
		XML_GENERAL_ENG_FUEL_PUMP_SWITCH,
		XML_GENERAL_ENG_FUEL_PUMP_ON,
		XML_GENERAL_ENG_RPM,
		XML_GENERAL_ENG_PCT_MAX_RPM,
		XML_GENERAL_ENG_MAX_REACHED_RPM,
		XML_GENERAL_ENG_THROTTLE_LEVER_POSITION,
		XML_GENERAL_ENG_MIXTURE_LEVER_POSITION,
		XML_GENERAL_ENG_PROPELLER_LEVER_POSITION,
		XML_GENERAL_ENG_STARTER,
		XML_GENERAL_ENG_EXHAUST_GAS_TEMPERATURE,
		XML_GENERAL_ENG_OIL_PRESSURE,
		XML_GENERAL_ENG_OIL_LEAKED_PERCENT,
		XML_GENERAL_ENG_COMBUSTION_SOUND_PERCENT,
		XML_GENERAL_ENG_DAMAGE_PERCENT,
		XML_GENERAL_ENG_OIL_TEMPERATURE,
		XML_GENERAL_ENG_FAILED,
		XML_GENERAL_ENG_GENERATOR_SWITCH,
		XML_GENERAL_ENG_GENERATOR_ACTIVE,
		XML_GENERAL_ENG_ANTI_ICE_POSITION,
		XML_GENERAL_ENG_FUEL_VALVE,
		XML_GENERAL_ENG_FUEL_PRESSURE,
		XML_GENERAL_ENG_ELAPSED_TIME,
		XML_RECIP_ENG_COWL_FLAP_POSITION,
		XML_RECIP_ENG_PRIMER,
		XML_RECIP_ENG_MANIFOLD_PRESSURE,
		XML_RECIP_ENG_ALTERNATE_AIR_POSITION,
		XML_RECIP_ENG_COOLANT_RESERVOIR_PERCENT,
		XML_RECIP_ENG_LEFT_MAGNETO,
		XML_RECIP_ENG_RIGHT_MAGNETO,
		XML_RECIP_ENG_BRAKE_POWER,
		XML_RECIP_ENG_STARTER_TORQUE,
		XML_RECIP_ENG_TURBOCHARGER_FAILED,
		XML_RECIP_ENG_EMERGENCY_BOOST_ACTIVE,
		XML_RECIP_ENG_EMERGENCY_BOOST_ELAPSED_TIME,
		XML_RECIP_ENG_WASTEGATE_POSITION,
		XML_RECIP_ENG_TURBINE_INLET_TEMPERATURE,
		XML_RECIP_ENG_CYLINDER_HEAD_TEMPERATURE,
		XML_RECIP_ENG_RADIATOR_TEMPERATURE,
		XML_RECIP_ENG_FUEL_AVAILABLE,
		XML_RECIP_ENG_FUEL_FLOW,
		XML_RECIP_ENG_FUEL_TANK_SELECTOR,
		XML_RECIP_ENG_FUEL_TANKS_USED,
		XML_RECIP_ENG_FUEL_NUMBER_TANKS_USED,
		XML_RECIP_CARBURETOR_TEMPERATURE,
		XML_RECIP_MIXTURE_RATIO,
		XML_TURB_ENG_N1,
		XML_TURB_ENG_N2,
		XML_TURB_ENG_CORRECTED_N1,
		XML_TURB_ENG_CORRECTED_N2,
		XML_TURB_ENG_CORRECTED_FF,
		XML_TURB_ENG_MAX_TORQUE_PERCENT,
		XML_TURB_ENG_PRESSURE_RATIO,
		XML_TURB_ENG_ITT,
		XML_TURB_ENG_AFTERBURNER,
		XML_TURB_ENG_JET_THRUST,
		XML_TURB_ENG_BLEED_AIR,
		XML_TURB_ENG_TANK_SELECTOR,
		XML_TURB_ENG_TANKS_USED,
		XML_TURB_ENG_NUM_TANKS_USED,
		XML_TURB_ENG_FUEL_FLOW_PPH,
		XML_TURB_ENG_FUEL_AVAILABLE,
		XML_TURB_ENG_PRIMARY_NOZZLE_PERCENT,
		XML_TURB_ENG_REVERSE_NOZZLE_PERCENT,
		XML_TURB_ENG_VIBRATION,
		XML_ENG_FAILED,
		XML_PARTIAL_PANEL_ADF,
		XML_PARTIAL_PANEL_AIRSPEED,
		XML_PARTIAL_PANEL_ALTIMETER,
		XML_PARTIAL_PANEL_ATTITUDE,
		XML_PARTIAL_PANEL_COMM,
		XML_PARTIAL_PANEL_COMPASS,
		XML_PARTIAL_PANEL_ELECTRICAL,
		XML_PARTIAL_PANEL_AVIONICS,
		XML_PARTIAL_PANEL_ENGINE,
		XML_PARTIAL_PANEL_FUEL_INDICATOR,
		XML_PARTIAL_PANEL_HEADING,
		XML_PARTIAL_PANEL_VERTICAL_VELOCITY,
		XML_PARTIAL_PANEL_TRANSPONDER,
		XML_PARTIAL_PANEL_NAV,
		XML_PARTIAL_PANEL_PITOT,
		XML_PARTIAL_PANEL_TURN_COORDINATOR,
		XML_PARTIAL_PANEL_VACUUM,
		XML_FUEL_TANK_CENTER_LEVEL,
		XML_FUEL_TANK_CENTER_CAPACITY,
		XML_FUEL_TANK_CENTER_QUANTITY,
		XML_FUEL_TANK_CENTER2_LEVEL,
		XML_FUEL_TANK_CENTER2_CAPACITY,
		XML_FUEL_TANK_CENTER2_QUANTITY,
		XML_FUEL_TANK_CENTER3_LEVEL,
		XML_FUEL_TANK_CENTER3_CAPACITY,
		XML_FUEL_TANK_CENTER3_QUANTITY,
		XML_FUEL_TANK_LEFT_MAIN_LEVEL,
		XML_FUEL_TANK_LEFT_MAIN_CAPACITY,
		XML_FUEL_TANK_LEFT_MAIN_QUANTITY,
		XML_FUEL_TANK_LEFT_AUX_LEVEL,
		XML_FUEL_TANK_LEFT_AUX_CAPACITY,
		XML_FUEL_TANK_LEFT_AUX_QUANTITY,
		XML_FUEL_TANK_LEFT_TIP_LEVEL,
		XML_FUEL_TANK_LEFT_TIP_CAPACITY,
		XML_FUEL_TANK_LEFT_TIP_QUANTITY,
		XML_FUEL_LEFT_QUANTITY,
		XML_FUEL_TANK_RIGHT_MAIN_LEVEL,
		XML_FUEL_TANK_RIGHT_MAIN_CAPACITY,
		XML_FUEL_TANK_RIGHT_MAIN_QUANTITY,
		XML_FUEL_TANK_RIGHT_AUX_LEVEL,
		XML_FUEL_TANK_RIGHT_AUX_CAPACITY,
		XML_FUEL_TANK_RIGHT_AUX_QUANTITY,
		XML_FUEL_TANK_RIGHT_TIP_LEVEL,
		XML_FUEL_TANK_RIGHT_TIP_CAPACITY,
		XML_FUEL_TANK_RIGHT_TIP_QUANTITY,
		XML_FUEL_RIGHT_QUANTITY,
		XML_FUEL_TANK_EXTERNAL1_LEVEL,
		XML_FUEL_TANK_EXTERNAL1_CAPACITY,
		XML_FUEL_TANK_EXTERNAL1_QUANTITY,
		XML_FUEL_TANK_EXTERNAL2_LEVEL,
		XML_FUEL_TANK_EXTERNAL2_CAPACITY,
		XML_FUEL_TANK_EXTERNAL2_QUANTITY,
		XML_FUEL_TOTAL_QUANTITY,
		XML_FUEL_TOTAL_CAPACITY,
		XML_FUEL_LEFT_CAPACITY,
		XML_FUEL_RIGHT_CAPACITY,
		XML_FUEL_WEIGHT_PER_GALLON,
		XML_FUEL_TANK_SELECTOR,
		XML_FUEL_CROSS_FEED,
		XML_NUM_FUEL_SELECTORS,
		XML_FUEL_SELECTED_QUANTITY_PERCENT,
		XML_FUEL_SELECTED_QUANTITY,
		XML_FUEL_TOTAL_QUANTITY_WEIGHT,
		XML_FUEL_SELECTED_TRANSFER_MODE,
		XML_WARNING_FUEL,
		XML_WARNING_FUEL_LEFT,
		XML_WARNING_FUEL_RIGHT,
		XML_WARNING_VACUUM,
		XML_WARNING_VACUUM_LEFT,
		XML_WARNING_VACUUM_RIGHT,
		XML_WARNING_OIL_PRESSURE,
		XML_WARNING_VOLTAGE,
		XML_WARNING_LOW_HEIGHT,
		XML_AUTOPILOT_AVAILABLE,
		XML_FLAPS_AVAILABLE,
		XML_STALL_HORN_AVAILABLE,
		XML_ENGINE_MIXURE_AVAILABLE,
		XML_CARB_HEAT_AVAILABLE,
		XML_SPOILER_AVAILABLE,
		XML_STROBES_AVAILABLE,
		XML_PROP_TYPE_AVAILABLE,
		XML_TOE_BRAKES_AVAILABLE,
		XML_IS_TAIL_DRAGGER,
		XML_SYSTEMS_AVAILABLE,
		XML_INSTRUMENTS_AVAILABLE,
		XML_FUEL_PUMP,
		XML_NAV_AVAILABLE,
		XML_ELECTRICAL_MASTER_BATTERY,
		XML_ELECTRICAL_OLD_CHARGING_AMPS,
		XML_ELECTRICAL_TOTAL_LOAD_AMPS,
		XML_ELECTRICAL_BATTERY_LOAD,
		XML_ELECTRICAL_BATTERY_VOLTAGE,
		XML_ELECTRICAL_MAIN_BUS_VOLTAGE,
		XML_ELECTRICAL_MAIN_BUS_AMPS,
		XML_ELECTRICAL_AVIONICS_BUS_VOLTAGE,
		XML_ELECTRICAL_AVIONICS_BUS_AMPS,
		XML_ELECTRICAL_HOT_BATTERY_BUS_VOLTAGE,
		XML_ELECTRICAL_HOT_BATTERY_BUS_AMPS,
		XML_ELECTRICAL_BATTERY_BUS_VOLTAGE,
		XML_ELECTRICAL_BATTERY_BUS_AMPS,
		XML_ELECTRICAL_GENALT_BUS_VOLTAGE,
		XML_ELECTRICAL_GENALT_BUS_AMPS,
		XML_CIRCUIT_GENERAL_PANEL_ON,
		XML_CIRCUIT_FLAP_MOTOR_ON,
		XML_CIRCUIT_GEAR_MOTOR_ON,
		XML_CIRCUIT_AUTOPILOT_ON,
		XML_CIRCUIT_AVIONICS_ON,
		XML_CIRCUIT_PITOT_HEAT_ON,
		XML_CIRCUIT_PROP_SYNC_ON,
		XML_CIRCUIT_AUTO_FEATHER_ON,
		XML_CIRCUIT_AUTO_BRAKES_ON,
		XML_CIRCUIT_STANDY_VACUUM_ON,
		XML_CIRCUIT_MARKER_BEACON_ON,
		XML_CIRCUIT_GEAR_WARNING_ON,
		XML_CIRCUIT_HYDRAULIC_PUMP_ON,
		XML_AMBIENT_DENSITY,
		XML_AMBIENT_TEMPERATURE,
		XML_AMBIENT_PRESSURE,
		XML_AMBIENT_WIND_VELOCITY,
		XML_AMBIENT_WIND_DIRECTION,
		XML_AMBIENT_WIND_X,
		XML_AMBIENT_WIND_Y,
		XML_AMBIENT_WIND_Z,
		XML_AMBIENT_PRECIP_STATE,
		XML_BAROMETER_PRESSURE,
		XML_SEA_LEVEL_PRESSURE,
		XML_TOTAL_AIR_TEMPERATURE,
		XML_STANDARD_ATM_TEMPERATURE,
		XML_AIRCRAFT_WIND_X,
		XML_AIRCRAFT_WIND_Y,
		XML_AIRCRAFT_WIND_Z,
		XML_HYDRAULIC_PRESSURE,
		XML_HYDRAULIC_RESERVOIR_PERCENT,
		XML_HYDRAULIC_SYSTEM_INTEGRITY,
		XML_GEAR_HYDRAULIC_PRESSURE,
		XML_CONCORDE_VISOR_NOSE_HANDLE,
		XML_CONCORDE_VISOR_POSITION_PERCENT,
		XML_CONCORDE_NOSE_ANGLE,
		XML_COM_TRANSMIT,
		XML_COM_RECIEVE_ALL,
		XML_NAV_SOUND,
		XML_DME_SOUND,
		XML_ADF_SOUND,
		XML_ADF_CARD,
		XML_MARKER_SOUND,
		XML_COM_ACTIVE_FREQUENCY,
		XML_COM_STANDBY_FREQUENCY,
		XML_COM_STATUS,
		XML_TRANSPONDER_CODE,
		XML_ADF_FREQUENCY,
		XML_ADF_EXT_FREQUENCY,
		XML_ADF_ACTIVE_FREQUENCY,
		XML_ADF_STANDBY_FREQUENCY,
		XML_ADF_LATLONALT,
		XML_ADF_SIGNAL,
		XML_ADF_RADIAL,
		XML_ADF_IDENT,
		XML_ADF_NAME,
		XML_NAV_ACTIVE_FREQUENCY,
		XML_NAV_STANDBY_FREQUENCY,
		XML_NAV_SIGNAL,
		XML_NAV_IDENT,
		XML_NAV_NAME,
		XML_NAV_CODES,
		XML_NAV_HAS_NAV,
		XML_NAV_HAS_LOCALIZER,
		XML_NAV_HAS_DME,
		XML_NAV_HAS_GLIDE_SLOPE,
		XML_NAV_BACK_COURSE_FLAGS,
		XML_NAV_MAGVAR,
		XML_NAV_RADIAL,
		XML_NAV_RADIAL_ERROR,
		XML_NAV_LOCALIZER,
		XML_NAV_GLIDE_SLOPE,
		XML_NAV_GLIDE_SLOPE_ERROR,
		XML_NAV_CDI,
		XML_NAV_GSI,
		XML_NAV_TOFROM,
		XML_NAV_GS_FLAG,
		XML_NAV_OBS,
		XML_NAV_DME,
		XML_NAV_DMESPEED,
		XML_NAV_VOR_LATLONALT,
		XML_NAV_GS_LATLONALT,
		XML_NAV_DME_LATLONALT,
		XML_MARKER_BEACON_STATE,
		XML_INNER_MARKER,
		XML_MIDDLE_MARKER,
		XML_OUTER_MARKER,
		XML_INNER_MARKER_LATLONALT,
		XML_MIDDLE_MARKER_LATLONALT,
		XML_OUTER_MARKER_LATLONALT,
		XML_REALISM,
		XML_AUTO_COORDINATION,
		XML_UNLIMITED_FUEL,
		XML_ATC_TYPE,
		XML_ATC_MODEL,
		XML_ATC_HEAVY,
		XML_ATC_ID,
		XML_ATC_AIRLINE,
		XML_ATC_FLIGHT_NUMBER,
		XML_STRUCT_LATLONALT,
		XML_STRUCT_LATLONALTPBH,
		XML_STRUCT_PBH32,
		XML_STRUCT_DAMAGEVISIBLE,
		XML_STRUCT_WORLDVELOCITY,
		XML_STRUCT_WORLD_ROTATION_VELOCITY,
		XML_STRUCT_ENGINE_POSITION,
		XML_PITOT_HEAT,
		XML_SMOKE_ENABLE,
		XML_SMOKESYSTEM_AVAILABLE,
		XML_G_FORCE,
		XML_MAX_G_FORCE,
		XML_MIN_G_FORCE,
		XML_SUCTION_PRESSURE,
		XML_RAD_INS_SWITCH,
		XML_TYPICAL_DESCENT_RATE,
		XML_VISUAL_MODEL_RADIUS,
		XML_IS_USER_SIM,
		XML_CONTROLLABLE,
		XML_HEADING_INDICATOR,
		XML_TITLE,
		XML_CATEGORY,
		XML_SIM_DISABLED,
		XML_PROP_DEICE_SWITCH,
		XML_STRUCTURAL_DEICE_SWITCH,
		XML_ARTIFICIAL_GROUND_ELEVATION,
		XML_SURFACE_TYPE,
		XML_SURFACE_CONDITION,
		XML_PUSHBACK_STATE,
		XML_HSI_CDI_NEEDLE,
		XML_HSI_GSI_NEEDLE,
		XML_HSI_CDI_NEEDLE_VALID,
		XML_HSI_GSI_NEEDLE_VALID,
		XML_HSI_TF_FLAGS,
		XML_HSI_BEARING,
		XML_HSI_BEARING_VALID,
		XML_HSI_HAS_LOCALIZER,
		XML_HSI_SPEED,
		XML_HSI_DISTANCE,
		XML_HSI_STATION_IDENT,
		XML_IS_SLEW_ACTIVE,
		XML_IS_SLEW_ALLOWED,
		XML_ATC_SUGGESTED_MIN_RWY_TAKEOFF,
		XML_ATC_SUGGESTED_MIN_RWY_LANDING,
		XML_YAW_STRING_ANGLE,
		XML_YAW_STRING_PCT_EXTENDED,
		XML_INDUCTOR_COMPASS_PERCENT_DEVIATION,
		XML_INDUCTOR_COMPASS_HEADING_REF,
		XML_ANEMOMETER_PCT_RPM,
		XML_GPS_POSITION_LAT,
		XML_GPS_POSITION_LON,
		XML_GPS_POSITION_ALT,
		XML_GPS_MAGVAR,
		XML_GPS_IS_ACTIVE_FLIGHT_PLAN,
		XML_GPS_IS_ACTIVE_WAY_POINT,
		XML_GPS_IS_ARRIVED,
		XML_GPS_IS_DIRECTTO_FLIGHTPLAN,
		XML_GPS_GROUND_SPEED,
		XML_GPS_GROUND_TRUE_HEADING,
		XML_GPS_GROUND_MAGNETIC_TRACK,
		XML_GPS_GROUND_TRUE_TRACK,
		XML_GPS_ETE,
		XML_GPS_ETA,
		XML_GPS_WP_DISTANCE,
		XML_GPS_WP_BEARING,
		XML_GPS_WP_TRUE_BEARING,
		XML_GPS_WP_CROSS_TRK,
		XML_GPS_WP_DESIRED_TRACK,
		XML_GPS_WP_TRUE_REQ_HDG,
		XML_GPS_WP_VERTICAL_SPEED,
		XML_GPS_WP_TRACK_ANGLE_ERROR,
		XML_GPS_WP_NEXT_ID,
		XML_GPS_WP_NEXT_LAT,
		XML_GPS_WP_NEXT_LON,
		XML_GPS_WP_NEXT_ALT,
		XML_GPS_WP_PREV_VALID,
		XML_GPS_WP_PREV_ID,
		XML_GPS_WP_PREV_LAT,
		XML_GPS_WP_PREV_LON,
		XML_GPS_WP_PREV_ALT,
		XML_GPS_WP_ETE,
		XML_GPS_WP_ETA,
		XML_GPS_COURSE_TO_STEER,
		XML_GPS_FLIGHT_PLAN_WP_INDEX,
		XML_GPS_FLIGHT_PLAN_WP_COUNT,
		XML_GPS_IS_ACTIVE_WP_LOCKED,
		XML_GPS_IS_APPROACH_LOADED,
		XML_GPS_IS_APPROACH_ACTIVE,
		XML_GPS_APPROACH_MODE,
		XML_GPS_APPROACH_WP_TYPE,
		XML_GPS_APPROACH_IS_WP_RUNWAY,
		XML_GPS_APPROACH_SEGMENT_TYPE,
		XML_GPS_APPROACH_AIRPORT_ID,
		XML_GPS_APPROACH_APPROACH_INDEX,
		XML_GPS_APPROACH_APPROACH_ID,
		XML_GPS_APPROACH_APPROACH_TYPE,
		XML_GPS_APPROACH_TRANSITION_INDEX,
		XML_GPS_APPROACH_TRANSITION_ID,
		XML_GPS_APPROACH_IS_FINAL,
		XML_GPS_APPROACH_IS_MISSED,
		XML_GPS_APPROACH_TIMEZONE_DEVIATION,
		XML_GPS_APPROACH_WP_INDEX,
		XML_GPS_APPROACH_WP_COUNT,
		XML_GPS_TARGET_DISTANCE,
		XML_GPS_TARGET_ALTITUDE,
		XML_USER_INPUT_ENABLED,
		XML_ROTOR_BRAKE_HANDLE_POS,
		XML_ROTOR_BRAKE_ACTIVE,
		XML_ROTOR_CLUTCH_SWITCH_POS,
		XML_ROTOR_CLUTCH_ACTIVE,
		XML_ROTOR_TEMPERATURE,
		XML_ROTOR_CHIP_DETECTED,
		XML_ROTOR_GOV_SWITCH_POS,
		XML_ROTOR_GOV_ACTIVE,
		XML_ROTOR_LATERAL_TRIM_PCT,
		XML_ROTOR_RPM_PCT,
		XML_GEAR_DAMAGE_BY_SPEED,
		XML_GEAR_SPEED_EXCEEDED,
		XML_FLAP_DAMAGE_BY_SPEED,
		XML_FLAP_SPEED_EXCEEDED,
		XML_ESTIMATED_CRUISE_SPEED,
		XML_ESTIMATED_FUEL_FLOW,
		XML_EYEPOINT_POSITION,
		XML_DEFAULT_2D_COCKPIT_VIEW_ANGLE,
		XML_DEFAULT_VC_COCKPIT_VIEW_ANGLE,
		XML_NAV_VOR_LLAF64,
		XML_NAV_GS_LLAF64,
		XML_NAV_RAW_GLIDE_SLOPE,
		XML_WINDSHIELD_RAIN_EFFECT_AVAILABLE,
		XML_TRAILING_EDGE_FLAPS0_LEFT_PERCENT,
		XML_TRAILING_EDGE_FLAPS1_LEFT_PERCENT,
		XML_TRAILING_EDGE_FLAPS0_RIGHT_PERCENT,
		XML_TRAILING_EDGE_FLAPS1_RIGHT_PERCENT,
		XML_LEADING_EDGE_FLAPS0_LEFT_PERCENT,
		XML_LEADING_EDGE_FLAPS1_LEFT_PERCENT,
		XML_LEADING_EDGE_FLAPS0_RIGHT_PERCENT,
		XML_LEADING_EDGE_FLAPS1_RIGHT_PERCENT,
		XML_TRAILING_EDGE_FLAPS0_LEFT_ANGLE,
		XML_TRAILING_EDGE_FLAPS1_LEFT_ANGLE,
		XML_TRAILING_EDGE_FLAPS0_RIGHT_ANGLE,
		XML_TRAILING_EDGE_FLAPS1_RIGHT_ANGLE,
		XML_LEADING_EDGE_FLAPS0_LEFT_ANGLE,
		XML_LEADING_EDGE_FLAPS1_LEFT_ANGLE,
		XML_LEADING_EDGE_FLAPS0_RIGHT_ANGLE,
		XML_LEADING_EDGE_FLAPS1_RIGHT_ANGLE,
		XML_ELEVON1_DEFLECTION,
		XML_ELEVON2_DEFLECTION,
		XML_ELEVON3_DEFLECTION,
		XML_ELEVON4_DEFLECTION,
		XML_ELEVON5_DEFLECTION,
		XML_ELEVON6_DEFLECTION,
		XML_ELEVON7_DEFLECTION,
		XML_ELEVON8_DEFLECTION,
		XML_PROP1_RPM,
		XML_PROP2_RPM,
		XML_PROP3_RPM,
		XML_PROP4_RPM,
		XML_PROP1_MAX_RPM_PERCENT,
		XML_PROP2_MAX_RPM_PERCENT,
		XML_PROP3_MAX_RPM_PERCENT,
		XML_PROP4_MAX_RPM_PERCENT,
		XML_PROP1_THRUST,
		XML_PROP2_THRUST,
		XML_PROP3_THRUST,
		XML_PROP4_THRUST,
		XML_PROP1_BETA,
		XML_PROP2_BETA,
		XML_PROP3_BETA,
		XML_PROP4_BETA,
		XML_PROP1_FEATHERING_INHIBIT,
		XML_PROP2_FEATHERING_INHIBIT,
		XML_PROP3_FEATHERING_INHIBIT,
		XML_PROP4_FEATHERING_INHIBIT,
		XML_PROP1_FEATHERED,
		XML_PROP2_FEATHERED,
		XML_PROP3_FEATHERED,
		XML_PROP4_FEATHERED,
		XML_PROP1_SYNC_DELTA_LEVER,
		XML_PROP2_SYNC_DELTA_LEVER,
		XML_PROP3_SYNC_DELTA_LEVER,
		XML_PROP4_SYNC_DELTA_LEVER,
		XML_PROP1_AUTO_FEATHER_ARMED,
		XML_PROP2_AUTO_FEATHER_ARMED,
		XML_PROP3_AUTO_FEATHER_ARMED,
		XML_PROP4_AUTO_FEATHER_ARMED,
		XML_ENG1_COMBUSTION,
		XML_ENG2_COMBUSTION,
		XML_ENG3_COMBUSTION,
		XML_ENG4_COMBUSTION,
		XML_OLD_ENG1_STARTER,
		XML_OLD_ENG2_STARTER,
		XML_OLD_ENG3_STARTER,
		XML_OLD_ENG4_STARTER,
		XML_ENG1_N1_RPM,
		XML_ENG2_N1_RPM,
		XML_ENG3_N1_RPM,
		XML_ENG4_N1_RPM,
		XML_ENG1_N2_RPM,
		XML_ENG2_N2_RPM,
		XML_ENG3_N2_RPM,
		XML_ENG4_N2_RPM,
		XML_ENG1_FUEL_FLOW_GPH,
		XML_ENG2_FUEL_FLOW_GPH,
		XML_ENG3_FUEL_FLOW_GPH,
		XML_ENG4_FUEL_FLOW_GPH,
		XML_ENG1_FUEL_FLOW_PPH,
		XML_ENG2_FUEL_FLOW_PPH,
		XML_ENG3_FUEL_FLOW_PPH,
		XML_ENG4_FUEL_FLOW_PPH,
		XML_ENG1_FUEL_FLOW_PPH_SSL,
		XML_ENG2_FUEL_FLOW_PPH_SSL,
		XML_ENG3_FUEL_FLOW_PPH_SSL,
		XML_ENG4_FUEL_FLOW_PPH_SSL,
		XML_ENG1_TORQUE,
		XML_ENG2_TORQUE,
		XML_ENG3_TORQUE,
		XML_ENG4_TORQUE,
		XML_ENG1_ANTI_ICE,
		XML_ENG2_ANTI_ICE,
		XML_ENG3_ANTI_ICE,
		XML_ENG4_ANTI_ICE,
		XML_ENG1_PRESSURE_RATIO,
		XML_ENG2_PRESSURE_RATIO,
		XML_ENG3_PRESSURE_RATIO,
		XML_ENG4_PRESSURE_RATIO,
		XML_ENG1_PRESSURE_RATIO_GES,
		XML_ENG2_PRESSURE_RATIO_GES,
		XML_ENG3_PRESSURE_RATIO_GES,
		XML_ENG4_PRESSURE_RATIO_GES,
		XML_ENG1_EXHAUST_GAS_TEMPERATURE,
		XML_ENG2_EXHAUST_GAS_TEMPERATURE,
		XML_ENG3_EXHAUST_GAS_TEMPERATURE,
		XML_ENG4_EXHAUST_GAS_TEMPERATURE,
		XML_ENG1_EXHAUST_GAS_TEMPERATURE_GES,
		XML_ENG2_EXHAUST_GAS_TEMPERATURE_GES,
		XML_ENG3_EXHAUST_GAS_TEMPERATURE_GES,
		XML_ENG4_EXHAUST_GAS_TEMPERATURE_GES,
		XML_ENG1_CYLINDER_HEAD_TEMPERATURE,
		XML_ENG2_CYLINDER_HEAD_TEMPERATURE,
		XML_ENG3_CYLINDER_HEAD_TEMPERATURE,
		XML_ENG4_CYLINDER_HEAD_TEMPERATURE,
		XML_ENG1_OIL_TEMPERATURE,
		XML_ENG2_OIL_TEMPERATURE,
		XML_ENG3_OIL_TEMPERATURE,
		XML_ENG4_OIL_TEMPERATURE,
		XML_ENG1_OIL_PRESSURE,
		XML_ENG2_OIL_PRESSURE,
		XML_ENG3_OIL_PRESSURE,
		XML_ENG4_OIL_PRESSURE,
		XML_ENG1_OIL_QUANTITY,
		XML_ENG2_OIL_QUANTITY,
		XML_ENG3_OIL_QUANTITY,
		XML_ENG4_OIL_QUANTITY,
		XML_ENG1_HYDRAULIC_PRESSURE,
		XML_ENG2_HYDRAULIC_PRESSURE,
		XML_ENG3_HYDRAULIC_PRESSURE,
		XML_ENG4_HYDRAULIC_PRESSURE,
		XML_ENG1_HYDRAULIC_QUANTITY,
		XML_ENG2_HYDRAULIC_QUANTITY,
		XML_ENG3_HYDRAULIC_QUANTITY,
		XML_ENG4_HYDRAULIC_QUANTITY,
		XML_ENG1_MANIFOLD_PRESSURE,
		XML_ENG2_MANIFOLD_PRESSURE,
		XML_ENG3_MANIFOLD_PRESSURE,
		XML_ENG4_MANIFOLD_PRESSURE,
		XML_ENG1_VIBRATION,
		XML_ENG2_VIBRATION,
		XML_ENG3_VIBRATION,
		XML_ENG4_VIBRATION,
		XML_ENG1_RPM_SCALER,
		XML_ENG2_RPM_SCALER,
		XML_ENG3_RPM_SCALER,
		XML_ENG4_RPM_SCALER,
		XML_ENG1_TURBINE_TEMPERATURE,
		XML_ENG2_TURBINE_TEMPERATURE,
		XML_ENG3_TURBINE_TEMPERATURE,
		XML_ENG4_TURBINE_TEMPERATURE,
		XML_ENG1_TORQUE_PERCENT,
		XML_ENG2_TORQUE_PERCENT,
		XML_ENG3_TORQUE_PERCENT,
		XML_ENG4_TORQUE_PERCENT,
		XML_ENG1_FUEL_PRESSURE,
		XML_ENG2_FUEL_PRESSURE,
		XML_ENG3_FUEL_PRESSURE,
		XML_ENG4_FUEL_PRESSURE,
		XML_ENG1_ELECTRICAL_LOAD,
		XML_ENG2_ELECTRICAL_LOAD,
		XML_ENG3_ELECTRICAL_LOAD,
		XML_ENG4_ELECTRICAL_LOAD,
		XML_ENG1_TRANSMISSION_PRESSURE,
		XML_ENG2_TRANSMISSION_PRESSURE,
		XML_ENG3_TRANSMISSION_PRESSURE,
		XML_ENG4_TRANSMISSION_PRESSURE,
		XML_ENG1_TRANSMISSION_TEMPERATURE,
		XML_ENG2_TRANSMISSION_TEMPERATURE,
		XML_ENG3_TRANSMISSION_TEMPERATURE,
		XML_ENG4_TRANSMISSION_TEMPERATURE,
		XML_ENG1_ROTOR_RPM,
		XML_ENG2_ROTOR_RPM,
		XML_ENG3_ROTOR_RPM,
		XML_ENG4_ROTOR_RPM,
		XML_ENG1_FUEL_FLOW_BUG_POSITION,
		XML_ENG2_FUEL_FLOW_BUG_POSITION,
		XML_ENG3_FUEL_FLOW_BUG_POSITION,
		XML_ENG4_FUEL_FLOW_BUG_POSITION,
		XML_ENG1_MAX_RPM,
		XML_ENG2_MAX_RPM,
		XML_ENG3_MAX_RPM,
		XML_ENG4_MAX_RPM,
		XML_ENG1_ON_FIRE,
		XML_ENG2_ON_FIRE,
		XML_ENG3_ON_FIRE,
		XML_ENG4_ON_FIRE,
		XML_GENERAL_ENG1_COMBUSTION,
		XML_GENERAL_ENG2_COMBUSTION,
		XML_GENERAL_ENG3_COMBUSTION,
		XML_GENERAL_ENG4_COMBUSTION,
		XML_GENERAL_ENG1_MASTER_ALTERNATOR,
		XML_GENERAL_ENG2_MASTER_ALTERNATOR,
		XML_GENERAL_ENG3_MASTER_ALTERNATOR,
		XML_GENERAL_ENG4_MASTER_ALTERNATOR,
		XML_GENERAL_ENG1_FUEL_PUMP_SWITCH,
		XML_GENERAL_ENG2_FUEL_PUMP_SWITCH,
		XML_GENERAL_ENG3_FUEL_PUMP_SWITCH,
		XML_GENERAL_ENG4_FUEL_PUMP_SWITCH,
		XML_GENERAL_ENG1_FUEL_PUMP_ON,
		XML_GENERAL_ENG2_FUEL_PUMP_ON,
		XML_GENERAL_ENG3_FUEL_PUMP_ON,
		XML_GENERAL_ENG4_FUEL_PUMP_ON,
		XML_GENERAL_ENG1_RPM,
		XML_GENERAL_ENG2_RPM,
		XML_GENERAL_ENG3_RPM,
		XML_GENERAL_ENG4_RPM,
		XML_GENERAL_ENG1_THROTTLE_LEVER_POSITION,
		XML_GENERAL_ENG2_THROTTLE_LEVER_POSITION,
		XML_GENERAL_ENG3_THROTTLE_LEVER_POSITION,
		XML_GENERAL_ENG4_THROTTLE_LEVER_POSITION,
		XML_GENERAL_ENG1_MIXTURE_LEVER_POSITION,
		XML_GENERAL_ENG2_MIXTURE_LEVER_POSITION,
		XML_GENERAL_ENG3_MIXTURE_LEVER_POSITION,
		XML_GENERAL_ENG4_MIXTURE_LEVER_POSITION,
		XML_GENERAL_ENG1_PROPELLER_LEVER_POSITION,
		XML_GENERAL_ENG2_PROPELLER_LEVER_POSITION,
		XML_GENERAL_ENG3_PROPELLER_LEVER_POSITION,
		XML_GENERAL_ENG4_PROPELLER_LEVER_POSITION,
		XML_GENERAL_ENG1_STARTER,
		XML_GENERAL_ENG2_STARTER,
		XML_GENERAL_ENG3_STARTER,
		XML_GENERAL_ENG4_STARTER,
		XML_GENERAL_ENG1_EXHAUST_GAS_TEMPERATURE,
		XML_GENERAL_ENG2_EXHAUST_GAS_TEMPERATURE,
		XML_GENERAL_ENG3_EXHAUST_GAS_TEMPERATURE,
		XML_GENERAL_ENG4_EXHAUST_GAS_TEMPERATURE,
		XML_GENERAL_ENG1_OIL_PRESSURE,
		XML_GENERAL_ENG2_OIL_PRESSURE,
		XML_GENERAL_ENG3_OIL_PRESSURE,
		XML_GENERAL_ENG4_OIL_PRESSURE,
		XML_GENERAL_ENG1_OIL_LEAKED_PERCENT,
		XML_GENERAL_ENG2_OIL_LEAKED_PERCENT,
		XML_GENERAL_ENG3_OIL_LEAKED_PERCENT,
		XML_GENERAL_ENG4_OIL_LEAKED_PERCENT,
		XML_GENERAL_ENG1_COMBUSTION_SOUND_PERCENT,
		XML_GENERAL_ENG2_COMBUSTION_SOUND_PERCENT,
		XML_GENERAL_ENG3_COMBUSTION_SOUND_PERCENT,
		XML_GENERAL_ENG4_COMBUSTION_SOUND_PERCENT,
		XML_GENERAL_ENG1_DAMAGE_PERCENT,
		XML_GENERAL_ENG2_DAMAGE_PERCENT,
		XML_GENERAL_ENG3_DAMAGE_PERCENT,
		XML_GENERAL_ENG4_DAMAGE_PERCENT,
		XML_GENERAL_ENG1_OIL_TEMPERATURE,
		XML_GENERAL_ENG2_OIL_TEMPERATURE,
		XML_GENERAL_ENG3_OIL_TEMPERATURE,
		XML_GENERAL_ENG4_OIL_TEMPERATURE,
		XML_GENERAL_ENG1_FAILED,
		XML_GENERAL_ENG2_FAILED,
		XML_GENERAL_ENG3_FAILED,
		XML_GENERAL_ENG4_FAILED,
		XML_GENERAL_ENG1_GENERATOR_SWITCH,
		XML_GENERAL_ENG2_GENERATOR_SWITCH,
		XML_GENERAL_ENG3_GENERATOR_SWITCH,
		XML_GENERAL_ENG4_GENERATOR_SWITCH,
		XML_GENERAL_ENG1_GENERATOR_ACTIVE,
		XML_GENERAL_ENG2_GENERATOR_ACTIVE,
		XML_GENERAL_ENG3_GENERATOR_ACTIVE,
		XML_GENERAL_ENG4_GENERATOR_ACTIVE,
		XML_GENERAL_ENG1_ANTI_ICE_POSITION,
		XML_GENERAL_ENG2_ANTI_ICE_POSITION,
		XML_GENERAL_ENG3_ANTI_ICE_POSITION,
		XML_GENERAL_ENG4_ANTI_ICE_POSITION,
		XML_GENERAL_ENG1_FUEL_VALVE,
		XML_GENERAL_ENG2_FUEL_VALVE,
		XML_GENERAL_ENG3_FUEL_VALVE,
		XML_GENERAL_ENG4_FUEL_VALVE,
		XML_GENERAL_ENG1_FUEL_PRESSURE,
		XML_GENERAL_ENG2_FUEL_PRESSURE,
		XML_GENERAL_ENG3_FUEL_PRESSURE,
		XML_GENERAL_ENG4_FUEL_PRESSURE,
		XML_RECIP_ENG1_COWL_FLAP_POSITION,
		XML_RECIP_ENG2_COWL_FLAP_POSITION,
		XML_RECIP_ENG3_COWL_FLAP_POSITION,
		XML_RECIP_ENG4_COWL_FLAP_POSITION,
		XML_RECIP_ENG1_PRIMER,
		XML_RECIP_ENG2_PRIMER,
		XML_RECIP_ENG3_PRIMER,
		XML_RECIP_ENG4_PRIMER,
		XML_RECIP_ENG1_MANIFOLD_PRESSURE,
		XML_RECIP_ENG2_MANIFOLD_PRESSURE,
		XML_RECIP_ENG3_MANIFOLD_PRESSURE,
		XML_RECIP_ENG4_MANIFOLD_PRESSURE,
		XML_RECIP_ENG1_ALTERNATE_AIR_POSITION,
		XML_RECIP_ENG2_ALTERNATE_AIR_POSITION,
		XML_RECIP_ENG3_ALTERNATE_AIR_POSITION,
		XML_RECIP_ENG4_ALTERNATE_AIR_POSITION,
		XML_RECIP_ENG1_COOLANT_RESERVOIR_PERCENT,
		XML_RECIP_ENG2_COOLANT_RESERVOIR_PERCENT,
		XML_RECIP_ENG3_COOLANT_RESERVOIR_PERCENT,
		XML_RECIP_ENG4_COOLANT_RESERVOIR_PERCENT,
		XML_RECIP_ENG1_LEFT_MAGNETO,
		XML_RECIP_ENG2_LEFT_MAGNETO,
		XML_RECIP_ENG3_LEFT_MAGNETO,
		XML_RECIP_ENG4_LEFT_MAGNETO,
		XML_RECIP_ENG1_RIGHT_MAGNETO,
		XML_RECIP_ENG2_RIGHT_MAGNETO,
		XML_RECIP_ENG3_RIGHT_MAGNETO,
		XML_RECIP_ENG4_RIGHT_MAGNETO,
		XML_RECIP_ENG1_BRAKE_POWER,
		XML_RECIP_ENG2_BRAKE_POWER,
		XML_RECIP_ENG3_BRAKE_POWER,
		XML_RECIP_ENG4_BRAKE_POWER,
		XML_RECIP_ENG1_STARTER_TORQUE,
		XML_RECIP_ENG2_STARTER_TORQUE,
		XML_RECIP_ENG3_STARTER_TORQUE,
		XML_RECIP_ENG4_STARTER_TORQUE,
		XML_RECIP_ENG1_TURBOCHARGER_FAILED,
		XML_RECIP_ENG2_TURBOCHARGER_FAILED,
		XML_RECIP_ENG3_TURBOCHARGER_FAILED,
		XML_RECIP_ENG4_TURBOCHARGER_FAILED,
		XML_RECIP_ENG1_EMERGENCY_BOOST_ACTIVE,
		XML_RECIP_ENG2_EMERGENCY_BOOST_ACTIVE,
		XML_RECIP_ENG3_EMERGENCY_BOOST_ACTIVE,
		XML_RECIP_ENG4_EMERGENCY_BOOST_ACTIVE,
		XML_RECIP_ENG1_EMERGENCY_BOOST_ELAPSED_TIME,
		XML_RECIP_ENG2_EMERGENCY_BOOST_ELAPSED_TIME,
		XML_RECIP_ENG3_EMERGENCY_BOOST_ELAPSED_TIME,
		XML_RECIP_ENG4_EMERGENCY_BOOST_ELAPSED_TIME,
		XML_RECIP_ENG1_WASTEGATE_POSITION,
		XML_RECIP_ENG2_WASTEGATE_POSITION,
		XML_RECIP_ENG3_WASTEGATE_POSITION,
		XML_RECIP_ENG4_WASTEGATE_POSITION,
		XML_RECIP_ENG1_TURBINE_INLET_TEMPERATURE,
		XML_RECIP_ENG2_TURBINE_INLET_TEMPERATURE,
		XML_RECIP_ENG3_TURBINE_INLET_TEMPERATURE,
		XML_RECIP_ENG4_TURBINE_INLET_TEMPERATURE,
		XML_RECIP_ENG1_CYLINDER_HEAD_TEMPERATURE,
		XML_RECIP_ENG2_CYLINDER_HEAD_TEMPERATURE,
		XML_RECIP_ENG3_CYLINDER_HEAD_TEMPERATURE,
		XML_RECIP_ENG4_CYLINDER_HEAD_TEMPERATURE,
		XML_RECIP_ENG1_RADIATOR_TEMPERATURE,
		XML_RECIP_ENG2_RADIATOR_TEMPERATURE,
		XML_RECIP_ENG3_RADIATOR_TEMPERATURE,
		XML_RECIP_ENG4_RADIATOR_TEMPERATURE,
		XML_RECIP_ENG1_FUEL_AVAILABLE,
		XML_RECIP_ENG2_FUEL_AVAILABLE,
		XML_RECIP_ENG3_FUEL_AVAILABLE,
		XML_RECIP_ENG4_FUEL_AVAILABLE,
		XML_RECIP_ENG1_FUEL_FLOW,
		XML_RECIP_ENG2_FUEL_FLOW,
		XML_RECIP_ENG3_FUEL_FLOW,
		XML_RECIP_ENG4_FUEL_FLOW,
		XML_RECIP_ENG1_FUEL_TANK_SELECTOR,
		XML_RECIP_ENG2_FUEL_TANK_SELECTOR,
		XML_RECIP_ENG3_FUEL_TANK_SELECTOR,
		XML_RECIP_ENG4_FUEL_TANK_SELECTOR,
		XML_RECIP_ENG1_FUEL_TANKS_USED,
		XML_RECIP_ENG2_FUEL_TANKS_USED,
		XML_RECIP_ENG3_FUEL_TANKS_USED,
		XML_RECIP_ENG4_FUEL_TANKS_USED,
		XML_RECIP_ENG1_FUEL_NUMBER_TANKS_USED,
		XML_RECIP_ENG2_FUEL_NUMBER_TANKS_USED,
		XML_RECIP_ENG3_FUEL_NUMBER_TANKS_USED,
		XML_RECIP_ENG4_FUEL_NUMBER_TANKS_USED,
		XML_TURB_ENG1_N1,
		XML_TURB_ENG2_N1,
		XML_TURB_ENG3_N1,
		XML_TURB_ENG4_N1,
		XML_TURB_ENG1_N2,
		XML_TURB_ENG2_N2,
		XML_TURB_ENG3_N2,
		XML_TURB_ENG4_N2,
		XML_TURB_ENG1_CORRECTED_N1,
		XML_TURB_ENG2_CORRECTED_N1,
		XML_TURB_ENG3_CORRECTED_N1,
		XML_TURB_ENG4_CORRECTED_N1,
		XML_TURB_ENG1_CORRECTED_N2,
		XML_TURB_ENG2_CORRECTED_N2,
		XML_TURB_ENG3_CORRECTED_N2,
		XML_TURB_ENG4_CORRECTED_N2,
		XML_TURB_ENG1_CORRECTED_FF,
		XML_TURB_ENG2_CORRECTED_FF,
		XML_TURB_ENG3_CORRECTED_FF,
		XML_TURB_ENG4_CORRECTED_FF,
		XML_TURB_ENG1_MAX_TORQUE_PERCENT,
		XML_TURB_ENG2_MAX_TORQUE_PERCENT,
		XML_TURB_ENG3_MAX_TORQUE_PERCENT,
		XML_TURB_ENG4_MAX_TORQUE_PERCENT,
		XML_TURB_ENG1_PRESSURE_RATIO,
		XML_TURB_ENG2_PRESSURE_RATIO,
		XML_TURB_ENG3_PRESSURE_RATIO,
		XML_TURB_ENG4_PRESSURE_RATIO,
		XML_TURB_ENG1_ITT,
		XML_TURB_ENG2_ITT,
		XML_TURB_ENG3_ITT,
		XML_TURB_ENG4_ITT,
		XML_TURB_ENG1_AFTERBURNER,
		XML_TURB_ENG2_AFTERBURNER,
		XML_TURB_ENG3_AFTERBURNER,
		XML_TURB_ENG4_AFTERBURNER,
		XML_TURB_ENG1_JET_THRUST,
		XML_TURB_ENG2_JET_THRUST,
		XML_TURB_ENG3_JET_THRUST,
		XML_TURB_ENG4_JET_THRUST,
		XML_TURB_ENG1_BLEED_AIR,
		XML_TURB_ENG2_BLEED_AIR,
		XML_TURB_ENG3_BLEED_AIR,
		XML_TURB_ENG4_BLEED_AIR,
		XML_TURB_ENG1_TANK_SELECTOR,
		XML_TURB_ENG2_TANK_SELECTOR,
		XML_TURB_ENG3_TANK_SELECTOR,
		XML_TURB_ENG4_TANK_SELECTOR,
		XML_TURB_ENG1_TANKS_USED,
		XML_TURB_ENG2_TANKS_USED,
		XML_TURB_ENG3_TANKS_USED,
		XML_TURB_ENG4_TANKS_USED,
		XML_TURB_ENG1_NUM_TANKS_USED,
		XML_TURB_ENG2_NUM_TANKS_USED,
		XML_TURB_ENG3_NUM_TANKS_USED,
		XML_TURB_ENG4_NUM_TANKS_USED,
		XML_TURB_ENG1_FUEL_FLOW_PPH,
		XML_TURB_ENG2_FUEL_FLOW_PPH,
		XML_TURB_ENG3_FUEL_FLOW_PPH,
		XML_TURB_ENG4_FUEL_FLOW_PPH,
		XML_TURB_ENG1_FUEL_AVAILABLE,
		XML_TURB_ENG2_FUEL_AVAILABLE,
		XML_TURB_ENG3_FUEL_AVAILABLE,
		XML_TURB_ENG4_FUEL_AVAILABLE,
		XML_TURB_ENG1_PRIMARY_NOZZLE_PERCENT,
		XML_TURB_ENG2_PRIMARY_NOZZLE_PERCENT,
		XML_TURB_ENG3_PRIMARY_NOZZLE_PERCENT,
		XML_TURB_ENG4_PRIMARY_NOZZLE_PERCENT,
		XML_TURB_ENG1_REVERSE_NOZZLE_PERCENT,
		XML_TURB_ENG2_REVERSE_NOZZLE_PERCENT,
		XML_TURB_ENG3_REVERSE_NOZZLE_PERCENT,
		XML_TURB_ENG4_REVERSE_NOZZLE_PERCENT,
		XML_TURB_ENG1_VIBRATION,
		XML_TURB_ENG2_VIBRATION,
		XML_TURB_ENG3_VIBRATION,
		XML_TURB_ENG4_VIBRATION,
		XML_ENG1_FAILED,
		XML_ENG2_FAILED,
		XML_ENG3_FAILED,
		XML_ENG4_FAILED,
		XML_PARTIAL_PANEL_ADF1,
		XML_PARTIAL_PANEL_COMM1,
		XML_PARTIAL_PANEL_COMM2,
		XML_PARTIAL_PANEL_TRANSPONDER1,
		XML_PARTIAL_PANEL_NAV1,
		XML_PARTIAL_PANEL_NAV2,
		XML_FUEL_TANK_SELECTOR_1,
		XML_FUEL_TANK_SELECTOR_2,
		XML_FUEL_TANK_SELECTOR_3,
		XML_FUEL_TANK_SELECTOR_4,
		XML_NAV1_AVAILABLE,
		XML_NAV2_AVAILABLE,
		XML_ELECTRICAL_GENALT1_BUS_VOLTAGE,
		XML_ELECTRICAL_GENALT2_BUS_VOLTAGE,
		XML_ELECTRICAL_GENALT3_BUS_VOLTAGE,
		XML_ELECTRICAL_GENALT4_BUS_VOLTAGE,
		XML_ELECTRICAL_GENALT1_BUS_AMPS,
		XML_ELECTRICAL_GENALT2_BUS_AMPS,
		XML_ELECTRICAL_GENALT3_BUS_AMPS,
		XML_ELECTRICAL_GENALT4_BUS_AMPS,
		XML_HYDRAULIC1_PRESSURE,
		XML_HYDRAULIC2_PRESSURE,
		XML_HYDRAULIC3_PRESSURE,
		XML_HYDRAULIC4_PRESSURE,
		XML_HYDRAULIC1_RESERVOIR_PERCENT,
		XML_HYDRAULIC2_RESERVOIR_PERCENT,
		XML_HYDRAULIC3_RESERVOIR_PERCENT,
		XML_HYDRAULIC4_RESERVOIR_PERCENT,
		XML_COM1_TRANSMIT,
		XML_COM2_TRANSMIT,
		XML_NAV1_SOUND,
		XML_NAV2_SOUND,
		XML_COM1_ACTIVE_FREQUENCY,
		XML_COM2_ACTIVE_FREQUENCY,
		XML_COM1_STANDBY_FREQUENCY,
		XML_COM2_STANDBY_FREQUENCY,
		XML_COM1_STATUS,
		XML_COM2_STATUS,
		XML_TRANSPONDER1_CODE,
		XML_ADF1_FREQUENCY,
		XML_ADF1_EXT_FREQUENCY,
		XML_ADF1_ACTIVE_FREQUENCY,
		XML_ADF1_STANDBY_FREQUENCY,
		XML_ADF1_LATLONALT,
		XML_ADF1_SIGNAL,
		XML_ADF1_RADIAL,
		XML_ADF1_IDENT,
		XML_ADF1_NAME,
		XML_NAV1_ACTIVE_FREQUENCY,
		XML_NAV2_ACTIVE_FREQUENCY,
		XML_NAV1_STANDBY_FREQUENCY,
		XML_NAV2_STANDBY_FREQUENCY,
		XML_NAV1_SIGNAL,
		XML_NAV2_SIGNAL,
		XML_NAV1_IDENT,
		XML_NAV2_IDENT,
		XML_NAV1_NAME,
		XML_NAV2_NAME,
		XML_NAV1_CODES,
		XML_NAV2_CODES,
		XML_NAV1_HAS_NAV,
		XML_NAV2_HAS_NAV,
		XML_NAV1_HAS_LOCALIZER,
		XML_NAV2_HAS_LOCALIZER,
		XML_NAV1_HAS_DME,
		XML_NAV2_HAS_DME,
		XML_NAV1_HAS_GLIDE_SLOPE,
		XML_NAV2_HAS_GLIDE_SLOPE,
		XML_NAV1_BACK_COURSE_FLAGS,
		XML_NAV2_BACK_COURSE_FLAGS,
		XML_NAV1_MAGVAR,
		XML_NAV2_MAGVAR,
		XML_NAV1_RADIAL,
		XML_NAV2_RADIAL,
		XML_NAV1_RADIAL_ERROR,
		XML_NAV2_RADIAL_ERROR,
		XML_NAV1_LOCALIZER,
		XML_NAV2_LOCALIZER,
		XML_NAV1_GLIDE_SLOPE,
		XML_NAV2_GLIDE_SLOPE,
		XML_NAV1_GLIDE_SLOPE_ERROR,
		XML_NAV2_GLIDE_SLOPE_ERROR,
		XML_NAV1_CDI,
		XML_NAV2_CDI,
		XML_NAV1_GSI,
		XML_NAV2_GSI,
		XML_NAV1_TOFROM,
		XML_NAV2_TOFROM,
		XML_NAV1_GS_FLAG,
		XML_NAV2_GS_FLAG,
		XML_NAV1_OBS,
		XML_NAV2_OBS,
		XML_NAV1_DME,
		XML_NAV2_DME,
		XML_NAV1_DMESPEED,
		XML_NAV2_DMESPEED,
		XML_NAV1_VOR_LATLONALT,
		XML_NAV2_VOR_LATLONALT,
		XML_NAV1_GS_LATLONALT,
		XML_NAV2_GS_LATLONALT,
		XML_NAV1_DME_LATLONALT,
		XML_NAV2_DME_LATLONALT,
		XML_HEADING_INDICATOR_0,
		XML_HEADING_INDICATOR_1,
		XML_HEADING_INDICATOR_2,
		XML_HEADING_INDICATOR_3,
		XML_HEADING_INDICATOR_4,
		GAUGE_XML_TOKEN
}XML_TOKEN;	


#endif//	TRAFFICDATA_H
